@@ -4,6 +4,7 @@ from pathlib import Path
 import fileinput
 import sys
 from src.utils.path_manager_singleton import PathManagerSingleton
+from src.utils.config_manager_singleton import ConfigManagerSingleton
 
 # list of subdomains is a set of strings, could be:
 # "test_project/tp_src/api"
@@ -23,6 +24,9 @@ def plantuml_diagram_creator_sub_domains(
 ):
     create_directory_if_not_exist(save_location)
 
+    global root_module
+    root_module = root_folder
+
     diagram_type = "package "
     diagram_name = diagram_name.replace(" ", "_")
     diagram_name_txt = ""
@@ -47,16 +51,23 @@ def plantuml_diagram_creator_sub_domains(
 
     # adding root to the drawing IF its meant to be in there
 
-    if check_if_module_should_be_in_filtered_graph(root_node.path, packages):
+    if check_if_module_should_be_in_filtered_graph(root_node, packages):
         f = open(diagram_name_txt, "a")
         f.write("@startuml \n")
-        f.write("title " + diagram_name + "\n")
-        f.write(diagram_type + root_node.name + "\n")
+        f.write("skinparam backgroundColor GhostWhite" + "\n")
+        f.write(
+            "title **Title**: "
+            + diagram_name
+            + ". **Rootfolder**: "
+            + root_folder
+            + "\n"
+        )
         f.close()
     else:
         f = open(diagram_name_txt, "a")
         f.write("@startuml \n")
-        f.write("title " + diagram_name + "\n")
+        f.write("skinparam backgroundColor GhostWhite" + "\n")
+        f.write("title " + diagram_name + " \n")
         f.close()
 
     while not que.isEmpty():
@@ -65,21 +76,27 @@ def plantuml_diagram_creator_sub_domains(
         # adds all modules we want in our subgraph
         for child in curr_node.child_module:
             if child.path not in node_tracker:
-                duplicate_name_check(name_tracker, child, node_tracker, None, True)
+                duplicate_name_check(
+                    name_tracker, child, node_tracker, None, True
+                )
                 que.enqueue(child)
                 node_tracker[child.path] = child
                 name_tracker[child.name] = child
 
-                if not ignore_modules_check(ignore_packages, child.path, root_folder):
+                if not ignore_modules_check(ignore_packages, child.path):
                     if check_if_module_should_be_in_filtered_graph(
-                        child.path, packages
+                        child, packages
                     ):
+                        config_manager = ConfigManagerSingleton()
+                        package_color = config_manager.package_color
                         f = open(diagram_name_txt, "a")
                         f.write(
                             diagram_type
                             + '"'
-                            + get_name_for_module_duplicate_checker(child, path_view)
-                            + '"'
+                            + get_name_for_module_duplicate_checker(
+                                child, path_view
+                            )
+                            + f'"#{package_color}'
                             + "\n"
                         )
                         f.close()
@@ -98,34 +115,39 @@ def plantuml_diagram_creator_sub_domains(
                 node_tracker_dependencies[child.path] = True
 
         dependencies: set[BTModule] = curr_node.get_module_dependencies()
-        name_curr_node = get_name_for_module_duplicate_checker(curr_node, path_view)
+        name_curr_node = get_name_for_module_duplicate_checker(
+            curr_node, path_view
+        )
 
-        if not ignore_modules_check(ignore_packages, curr_node.path, root_folder):
+        if not ignore_modules_check(ignore_packages, curr_node.path):
             for dependency in dependencies:
-                if not ignore_modules_check(
-                    ignore_packages, dependency.path, root_folder
-                ):
+                if not ignore_modules_check(ignore_packages, dependency.path):
                     name_dependency = get_name_for_module_duplicate_checker(
                         dependency, path_view
                     )
                     if check_if_module_should_be_in_filtered_graph(
-                        dependency.path, packages
+                        dependency, packages
                     ) and check_if_module_should_be_in_filtered_graph(
-                        curr_node.path, packages
+                        curr_node, packages
                     ):
                         # this if statement is made so that we dont point to ourselves
                         if name_curr_node != name_dependency:
                             # used to detect dependency changes
                             if name_curr_node in dependencies_map:
-                                dependency_list: list = dependencies_map[name_curr_node]
+                                dependency_list: list = dependencies_map[
+                                    name_curr_node
+                                ]
                                 dependency_list.append(dependency)
-                                dependencies_map[name_curr_node] = dependency_list
+                                dependencies_map[
+                                    name_curr_node
+                                ] = dependency_list
                             else:
                                 dependency_list = []
                                 dependency_list.append(dependency)
-                                dependencies_map[name_curr_node] = dependency_list
+                                dependencies_map[
+                                    name_curr_node
+                                ] = dependency_list
 
-                            ##
                             f = open(diagram_name_txt, "a")
                             f.write(
                                 '"'
@@ -134,7 +156,7 @@ def plantuml_diagram_creator_sub_domains(
                                 + "-->"
                                 + '"'
                                 + name_dependency
-                                + '"'
+                                + f'"{get_dependency_string(curr_node, dependency)}'
                                 + "\n"
                             )
                             f.close()
@@ -156,14 +178,12 @@ def plantuml_diagram_creator_sub_domains(
                 if child.path not in bfs_node_tracker:
                     que.enqueue(child)
                     bfs_node_tracker[child.path] = True
-                    if not ignore_modules_check(
-                        ignore_packages, child.path, root_folder
-                    ):
+                    if not ignore_modules_check(ignore_packages, child.path):
                         duplicate_name_check(
                             main_nodes, child, node_tracker, root_folder, True
                         )
                         if check_if_module_should_be_in_filtered_graph(
-                            child.path, packages, compare_graph_root
+                            child, packages
                         ):
 
                             main_nodes[child.name] = child
@@ -172,7 +192,7 @@ def plantuml_diagram_creator_sub_domains(
                             if (
                                 child.name not in name_tracker
                                 and not ignore_modules_check(
-                                    ignore_packages, child.path, root_folder
+                                    ignore_packages, child.path
                                 )
                             ):
                                 diff_checker = True
@@ -181,7 +201,7 @@ def plantuml_diagram_creator_sub_domains(
                                     diagram_type
                                     + '"'
                                     + get_name_for_module_duplicate_checker(
-                                        child, path_view, True
+                                        child, path_view
                                     )
                                     + '" #red'
                                     + "\n"
@@ -191,8 +211,8 @@ def plantuml_diagram_creator_sub_domains(
         for child in name_tracker.values():
             # children from original graph
             if check_if_module_should_be_in_filtered_graph(
-                child.path, packages
-            ) and not ignore_modules_check(ignore_packages, child.path, root_folder):
+                child, packages
+            ) and not ignore_modules_check(ignore_packages, child.path):
                 node: BTModule = child
                 name = node.name
                 if name not in main_nodes:
@@ -201,7 +221,9 @@ def plantuml_diagram_creator_sub_domains(
                     f.write(
                         diagram_type
                         + '"'
-                        + get_name_for_module_duplicate_checker(node, path_view, True)
+                        + get_name_for_module_duplicate_checker(
+                            node, path_view
+                        )
                         + '" #green'
                         + "\n"
                     )
@@ -222,12 +244,14 @@ def plantuml_diagram_creator_sub_domains(
                     que.enqueue(child)
                     node_tracker_dependencies[child.path] = True
 
-            if not ignore_modules_check(ignore_packages, curr_node.path, root_folder):
+            if not ignore_modules_check(ignore_packages, curr_node.path):
 
                 name_curr_node = get_name_for_module_duplicate_checker(
-                    curr_node, path_view, True
+                    curr_node, path_view
                 )
-                dependencies: set[BTModule] = curr_node.get_module_dependencies()
+                dependencies: set[
+                    BTModule
+                ] = curr_node.get_module_dependencies()
 
                 dependencies_map_main_graph[name_curr_node] = dependencies
 
@@ -237,15 +261,17 @@ def plantuml_diagram_creator_sub_domains(
 
                 for dependency in list_of_red_dependencies:
                     if not ignore_modules_check(
-                        ignore_packages, dependency.path, root_folder
+                        ignore_packages, dependency.path
                     ):
-                        name_dependency = get_name_for_module_duplicate_checker(
-                            dependency, path_view, True
+                        name_dependency = (
+                            get_name_for_module_duplicate_checker(
+                                dependency, path_view
+                            )
                         )
                         if check_if_module_should_be_in_filtered_graph(
-                            dependency.path, packages
+                            dependency, packages
                         ) and check_if_module_should_be_in_filtered_graph(
-                            curr_node.path, packages
+                            curr_node, packages
                         ):
                             # this if statement is made so that we dont point to ourselves
                             if name_curr_node != name_dependency:
@@ -258,7 +284,7 @@ def plantuml_diagram_creator_sub_domains(
                                     + "-->"
                                     + '"'
                                     + name_dependency
-                                    + '" #red'
+                                    + f'" #red {get_dependency_string(curr_node, dependency)}'
                                     + "\n"
                                 )
                                 f.close()
@@ -269,11 +295,14 @@ def plantuml_diagram_creator_sub_domains(
 
             list_of_old_dependencies = []
             if dependency in dependencies_map_main_graph:
-                list_of_old_dependencies = dependencies_map_main_graph[dependency]
+                list_of_old_dependencies = dependencies_map_main_graph[
+                    dependency
+                ]
 
             for dep in list_of_new_dependencies:
-                path_manager = PathManagerSingleton()
-                dep_name = path_manager.get_relative_path_from_project_root(dep.path)
+                dep_name = get_name_for_module_duplicate_checker(
+                    dep, path_view
+                )
                 if not path_view:
                     dep_name = dep.name
 
@@ -282,27 +311,20 @@ def plantuml_diagram_creator_sub_domains(
                     if dep.name == dep_old.name:
                         not_found_partner = False
                         break
+
                 if not_found_partner:
                     diff_checker = True
-                    for line in fileinput.input(diagram_name_txt, inplace=True):
+                    for line in fileinput.input(
+                        diagram_name_txt, inplace=True
+                    ):
+
                         print(
                             line.replace(  # TOOD: this replace does not work
-                                '"' + dependency + '"' + "-->" + '"' + dep_name + '"',
-                                '"'
-                                + dependency
-                                + '"'
-                                + "-->"
-                                + '"'
-                                + dep_name
-                                + '" #green',
+                                f'"{dependency}"-->"{dep_name}"',
+                                f'"{dependency}"-->"{dep_name}" #green',
                             ),
                             end="",
                         )
-                    # f = open(diagram_name_txt, "a")
-                    # f.write(
-                    #     '"' + dependency + '"' + "-->" + '"' + dep.name + '" #green' + "\n"
-                    # )
-                    # f.close()
 
     #########################################################################################################
     # ends the uml
@@ -320,7 +342,9 @@ def plantuml_diagram_creator_sub_domains(
         os.remove(diagram_name_txt)
 
 
-def find_red_dependencies(new_dependencies, node_name, old_dependencies: set[BTModule]):
+def find_red_dependencies(
+    new_dependencies, node_name, old_dependencies: set[BTModule]
+):
     res = []
 
     if node_name not in new_dependencies:
@@ -341,13 +365,21 @@ def find_red_dependencies(new_dependencies, node_name, old_dependencies: set[BTM
 
 def create_file(name):
     python_executable = sys.executable
-    os.system(python_executable + " -m plantuml " + name)
+    plantuml_server = os.getenv(
+        "PLANTUML_SERVER_URL",
+        "https://mt-plantuml-app-service.azurewebsites.net/img/",
+    )
+    os.system(
+        f"{python_executable} -m plantuml --server {plantuml_server}  {name}"
+    )
 
 
-def get_name_for_module_duplicate_checker(module: BTModule, path, diff_graph=False):
+def get_name_for_module_duplicate_checker(module: BTModule, path):
     if path:
         path_manager = PathManagerSingleton()
-        module_split = path_manager.get_relative_path_from_project_root(module.path)
+        module_split = path_manager.get_relative_path_from_project_root(
+            module.path, True
+        )
         return module_split
     if module.name_if_duplicate_exists is not None:
         return module.name_if_duplicate_exists
@@ -366,13 +398,17 @@ def duplicate_name_check(
         if not curr_node.name_if_duplicate_exists:
             if curr_node.name in node_names:
                 curr_node_split = split_path(curr_node.path)
-                curr_node_name = curr_node_split[-2] + "/" + curr_node_split[-1]
+                curr_node_name = (
+                    curr_node_split[-2] + "/" + curr_node_split[-1]
+                )
                 curr_node.name_if_duplicate_exists = curr_node_name
     else:
         if first:
             if curr_node.name in node_names:
                 curr_node_split = split_path(curr_node.path)
-                curr_node_name = curr_node_split[-2] + "/" + curr_node_split[-1]
+                curr_node_name = (
+                    curr_node_split[-2] + "/" + curr_node_split[-1]
+                )
                 curr_node.name_if_duplicate_exists = curr_node_name
 
 
@@ -389,9 +425,7 @@ def was_node_in_original_graph(node: BTModule, path_tracker, root_folder):
         return False
 
 
-def ignore_modules_check(
-    list_ignore: list[str], module, root_folder
-):  # TODO: remove root_folder if we go this direction
+def ignore_modules_check(list_ignore: list[str], module):
     path_manager = PathManagerSingleton()
     module = path_manager.get_relative_path_from_project_root(module, True)
     for ignore_package in list_ignore:
@@ -433,17 +467,50 @@ def old_ignore_modules_check(list_ignore, module, root_folder):
     return False
 
 
+# tp_src/api
+def check_if_allowed_module_is_root(allowed_module):
+    if type(allowed_module) == str:
+        if "/" in allowed_module:
+            splits = allowed_module.split("/")
+            if splits[0] == root_module and splits[1] == root_module:
+                return splits[0]
+    else:
+        if "/" in allowed_module["packagePath"]:
+            splits = allowed_module["packagePath"].split("/")
+            if splits[0] == root_module and splits[1] == root_module:
+                allowed_module["packagePath"] = splits[0]
+                return allowed_module
+    return allowed_module
+
+
 def check_if_module_should_be_in_filtered_graph(
-    module, allowed_modules, compare_graph_root=None
+    module: BTModule, allowed_modules
 ):
     if len(allowed_modules) == 0:
         return True
-    if compare_graph_root is not None:
-        path_manager = PathManagerSingleton()
-        module = path_manager.get_relative_path_from_project_root(module)
+    path_manager = PathManagerSingleton()
+    module_path = path_manager.get_relative_path_from_project_root(module.path)
     for module_curr in allowed_modules:
-        if module_curr in module:
-            return True
+        module_curr = check_if_allowed_module_is_root(module_curr)
+        # this means that we allow all of the sub system, no depth
+        if type(module_curr) == str:
+            if module_curr in module_path:
+                return True
+        # if we get here, it means that it is a specified object, at which point we must check for depth
+        else:
+            path = module_curr["packagePath"]
+            depth = module_curr["depth"]
+            if path == module_path:
+                module.depth = depth
+                return True
+            else:
+                if module.parent_module is not None:
+                    if module.parent_module.depth is not None:
+                        if path in module_path:
+                            if module.parent_module.depth > 0:
+                                module.depth = module.parent_module.depth - 1
+                                return True
+
     return False
 
 
@@ -466,6 +533,14 @@ def split_path(path) -> list[str]:
     return directories
 
 
+def get_dependency_string(module: BTModule, dependency_module: BTModule):
+    config_manager = ConfigManagerSingleton()
+    if config_manager.show_dependency_count:
+        dependency_count = module.get_dependency_count(dependency_module)
+        return f": {dependency_count}"
+    return ""
+
+
 class Queue:
     def __init__(self):
         self.items = []
